@@ -48,17 +48,16 @@ class Receiver:
     number = 0xFF
     kind = None
 
-    def __init__(self, handle, device_info):
+    def __init__(self, handle, path, product_id):
         assert handle
-        self.handle = handle
-        assert device_info
-        self.path = device_info.path
         self.isDevice = False  # some devices act as receiver so we need a property to distinguish them
-        # USB product id, used for some Nano receivers
-        self.product_id = device_info.product_id
+        self.handle = handle
+        self.path = path
+        self.product_id = product_id
         product_info = _product_information(self.product_id)
         if not product_info:
-            raise Exception('Unknown receiver type', self.product_id)
+            _log.warning('Unknown receiver type: %s', self.product_id)
+            product_info = {}
         self.receiver_kind = product_info.get('receiver_kind', 'unknown')
 
         # read the serial immediately, so we can find out max_devices
@@ -74,23 +73,17 @@ class Receiver:
                 self.max_devices = ord(serial_reply[6:7])
                 if self.max_devices <= 0 or self.max_devices > 6:
                     self.max_devices = product_info.get('max_devices', 1)
-                # TODO _properly_ figure out which receivers do and which don't support unpairing
-                # This code supposes that receivers that don't unpair support a pairing request for device index 0
-                if 'unpair' in product_info:
-                    self.may_unpair = product_info['unpair']
-                else:
-                    self.may_unpair = self.write_register(_R.receiver_pairing) is None
+                self.may_unpair = product_info.get('may_unpair', False)
             else:  # handle receivers that don't have a serial number specially (i.e., c534 and Bolt receivers)
                 self.serial = None
                 self.max_devices = product_info.get('max_devices', 1)
                 self.may_unpair = product_info.get('may_unpair', False)
 
-        self.name = product_info.get('name', '')
+        self.name = product_info.get('name', 'Receiver')
         self.re_pairs = product_info.get('re_pairs', False)
         self._str = '<%s(%s,%s%s)>' % (
             self.name.replace(' ', ''), self.path, '' if isinstance(self.handle, int) else 'T', self.handle
         )
-        self.ex100_27mhz_wpid_fix = product_info.get('ex100_27mhz_wpid_fix', False)
 
         self._firmware = None
         self._devices = {}
@@ -168,29 +161,29 @@ class Receiver:
                 return wpid, kind, 0
             else:
                 raise _base.NoSuchDevice(number=n, receiver=self, error='read Bolt wpid')
+        wpid = 0
+        kind = None
+        polling_rate = None
         pair_info = self.read_register(_R.receiver_info, _IR.pairing_information + n - 1)
-        polling_rate = 0
         if pair_info:  # may be either a Unifying receiver, or an Unifying-ready receiver
             wpid = _strhex(pair_info[3:5])
             kind = _hidpp10.DEVICE_KIND[ord(pair_info[7:8]) & 0x0F]
             polling_rate = ord(pair_info[2:3])
-        elif self.ex100_27mhz_wpid_fix:  # 27Mhz receiver, fill extracting WPID from udev path
+        elif self.receiver_kind == '27Mz':  # 27Mhz receiver, fill extracting WPID from udev path
             wpid = _hid.find_paired_node_wpid(self.path, n)
             if not wpid:
                 _log.error('Unable to get wpid from udev for device %d of %s', n, self)
                 raise _base.NoSuchDevice(number=n, receiver=self, error='Not present 27Mhz device')
-            kind = _hidpp10.DEVICE_KIND[self.get_kind_from_index(n, self)]
-        else:
-            # unifying protocol not supported, probably an old Nano receiver
+            kind = _hidpp10.DEVICE_KIND[self.get_kind_from_index(n)]
+        elif not self.receiver_kind == 'unifying':  # unifying protocol not supported, may be an old Nano receiver
             device_info = self.read_register(_R.receiver_info, 0x04)
-            if device_info is None:
-                _log.error('failed to read Nano wpid for device %d of %s', n, self)
-                raise _base.NoSuchDevice(number=n, receiver=self, error='read Nano wpid')
-            wpid = _strhex(device_info[3:5])
-            kind = _hidpp10.DEVICE_KIND[0x00]  # unknown kind
+            if device_info:
+                wpid = _strhex(device_info[3:5])
+                kind = _hidpp10.DEVICE_KIND[0x00]  # unknown kind
         return wpid, kind, polling_rate
 
     def device_extended_pairing_information(self, n):
+        serial = None
         power_switch = '(unknown)'
         if self.receiver_kind == 'bolt':
             pair_info = self.read_register(_R.receiver_info, _IR.bolt_pairing_information + n)
@@ -206,8 +199,6 @@ class Receiver:
             pair_info = self.read_register(0x2D5)
         if pair_info:
             serial = _strhex(pair_info[1:5])
-        else:  # fallback...
-            serial = self.serial
         return serial, power_switch
 
     def get_kind_from_index(self, index):
@@ -243,7 +234,6 @@ class Receiver:
 
         try:
             dev = Device(self, number, notification)
-            assert dev.wpid
             if _log.isEnabledFor(_INFO):
                 _log.info('%s: found new device %d (%s)', self, number, dev.wpid)
             self._devices[number] = dev
@@ -295,12 +285,17 @@ class Receiver:
     write_register = _hidpp10.write_register
 
     def __iter__(self):
-        for number in range(1, 1 + self.max_devices):
+        connected_devices = self.count()
+        found_devices = 0
+        for number in range(1, 8):  # some receivers have devices past their max # devices
+            if found_devices >= connected_devices:
+                return
             if number in self._devices:
                 dev = self._devices[number]
             else:
                 dev = self.__getitem__(number)
             if dev is not None:
+                found_devices += 1
                 yield dev
 
     def __getitem__(self, key):
@@ -313,7 +308,7 @@ class Receiver:
 
         if not isinstance(key, int):
             raise TypeError('key must be an integer')
-        if key < 1 or key > self.max_devices:
+        if key < 1 or key > 15:  # some receivers have devices past their max # devices
             raise IndexError(key)
 
         return self.register_new_device(key)
@@ -391,7 +386,7 @@ class Receiver:
         try:
             handle = _base.open_path(device_info.path)
             if handle:
-                return Receiver(handle, device_info)
+                return Receiver(handle, device_info.path, device_info.product_id)
         except OSError as e:
             _log.exception('open %s', device_info)
             if e.errno == _errno.EACCES:
