@@ -1,5 +1,4 @@
 # -*- python-mode -*-
-# -*- coding: UTF-8 -*-
 
 ## Copyright (C) 2020
 ##
@@ -17,8 +16,6 @@
 ## with this program; if not, write to the Free Software Foundation, Inc.,
 ## 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 from logitech_receiver import base as _base
 from logitech_receiver import hidpp10 as _hidpp10
 from logitech_receiver.common import strhex as _strhex
@@ -55,14 +52,14 @@ def run(receivers, args, find_receiver, _ignore):
     rgst = receiver.read_register(_R.devices_activity)
     print('    Device Activity       %#04x: %s' % (_R.devices_activity % 0x100, '0x' + _strhex(rgst) if rgst else 'None'))
 
-    for sub_reg in range(0, 6):
+    for sub_reg in range(0, 16):
         rgst = receiver.read_register(_R.receiver_info, sub_reg)
         print(
             '    Pairing Register %#04x %#04x: %s' %
             (_R.receiver_info % 0x100, sub_reg, '0x' + _strhex(rgst) if rgst else 'None')
         )
-    for device in range(0, 6):
-        for sub_reg in [0x10, 0x20, 0x30]:
+    for device in range(0, 7):
+        for sub_reg in [0x10, 0x20, 0x30, 0x50]:
             rgst = receiver.read_register(_R.receiver_info, sub_reg + device)
             print(
                 '    Pairing Register %#04x %#04x: %s' %
@@ -73,7 +70,14 @@ def run(receivers, args, find_receiver, _ignore):
             '    Pairing Name     %#04x %#02x: %s' %
             (_R.receiver_info % 0x100, 0x40 + device, rgst[2:2 + ord(rgst[1:2])] if rgst else 'None')
         )
-
+        for part in range(1, 4):
+            rgst = receiver.read_register(_R.receiver_info, 0x60 + device, part)
+            print(
+                '    Pairing Name     %#04x %#02x %#02x: %2d %s' % (
+                    _R.receiver_info % 0x100, 0x60 + device, part, ord(rgst[2:3]) if rgst else 0,
+                    rgst[3:3 + ord(rgst[2:3])] if rgst else 'None'
+                )
+            )
     for sub_reg in range(0, 5):
         rgst = receiver.read_register(_R.firmware, sub_reg)
         print(
